/**
 * 5A Lab Power Supply With Digital Readout
 * Kerry Wong
 * http://www.kerrywong.com
 * 4/2012
 */
 
#include <AD770X.h>
#include <LiquidCrystal.h>

const int pinRS = 2;
const int pinEn = 3;

const int pinD4 = 4;
const int pinD5 = 5;
const int pinD6 = 6;
const int pinD7 = 7;

const double V_COEFF = 0.977 / (0.977 + 9.804); //voltage divider measured values
const double I_COEFF = 2.38; //depending on current sensing resistor value
const double I_OFFSET = 0.004; //idle current

char vDispMask[] = "  V =%6s (V) ";
char iDispMask[] = "  I =%6s (A) ";

LiquidCrystal lcd(pinRS, pinEn, pinD4, pinD5, pinD6, pinD7);
AD770X ad7706(2.5);

void displayInfo()
{
  double v, i, t1, t2;
  char tmp[7], buf[16];

  t1 = ad7706.readADResult(AD770X::CHN_AIN1);
  Serial.print(t1);

  t2 = ad7706.readADResult(AD770X::CHN_AIN2);
  Serial.print(" : ");
  Serial.println(t2);

  v = t1 / V_COEFF;  

  dtostrf(v, 5, 3, tmp);
  sprintf(buf, vDispMask, tmp);

  lcd.setCursor(0,0);
  lcd.print(buf);  
  
  i = t2 * I_COEFF - I_OFFSET;
  if (i < 0) i = 0;

  dtostrf(i, 5, 3, tmp);
  sprintf(buf, iDispMask, tmp);

  lcd.setCursor(0,1);
  lcd.print(buf);  
}

void setup()
{
  Serial.begin(9600);

  lcd.begin(16, 2);
  lcd.clear();

  ad7706.reset();
  ad7706.init(AD770X::CHN_AIN1);  
  ad7706.init(AD770X::CHN_AIN2);
}

void loop()
{
  displayInfo();
}

